@@ -40,7 +40,7 @@ static void      clearlooks_rc_style_merge        (GtkRcStyle             *dest,
 						   GtkRcStyle             *src);
 
 
-static GtkRcStyleClass *parent_class;
+static GtkRcStyleClass *clearlooks_parent_rc_class;
 
 GType clearlooks_type_rc_style = 0;
 
@@ -54,6 +54,11 @@ enum
   TOKEN_MENUITEMSTYLE,
   TOKEN_LISTVIEWITEMSTYLE,
   TOKEN_ANIMATION,
+  TOKEN_STYLE,
+  TOKEN_RADIUS,
+  
+  TOKEN_CLASSIC,
+  TOKEN_GLOSSY,
   
   TOKEN_TRUE,
   TOKEN_FALSE
@@ -64,7 +69,7 @@ static struct
     const gchar        *name;
     guint               token;
   }
-theme_symbols[] =
+clearlooks_gtk2_rc_symbols[] =
 {
   { "scrollbar_color",   TOKEN_SCROLLBARCOLOR  },
   { "contrast", 		 TOKEN_CONTRAST  },
@@ -73,7 +78,12 @@ theme_symbols[] =
   { "menubarstyle",      TOKEN_MENUBARSTYLE },
   { "menuitemstyle",     TOKEN_MENUITEMSTYLE },
   { "listviewitemstyle", TOKEN_LISTVIEWITEMSTYLE },
-  { "animation",	TOKEN_ANIMATION },
+  { "animation",         TOKEN_ANIMATION },
+  { "style",             TOKEN_STYLE },
+  { "radius",            TOKEN_RADIUS },
+
+  { "CLASSIC",           TOKEN_CLASSIC },
+  { "GLOSSY",            TOKEN_GLOSSY },
   
   { "TRUE",	TOKEN_TRUE },
   { "FALSE",	TOKEN_FALSE }
@@ -106,14 +116,15 @@ clearlooks_rc_style_register_type (GTypeModule *module)
 static void
 clearlooks_rc_style_init (ClearlooksRcStyle *clearlooks_rc)
 {
-  clearlooks_rc->has_scrollbar_color = FALSE;
+  clearlooks_rc->style = CL_STYLE_CLASSIC;
+
+  clearlooks_rc->flags = 0;
+  
   clearlooks_rc->contrast = 1.0;
-  clearlooks_rc->sunkenmenubar = 1;
-  clearlooks_rc->progressbarstyle = 0;
   clearlooks_rc->menubarstyle = 0;
-  clearlooks_rc->menuitemstyle = 1;
-  clearlooks_rc->listviewitemstyle = 1;
   clearlooks_rc->animation = FALSE;
+  clearlooks_rc->style = CL_STYLE_CLASSIC;
+  clearlooks_rc->radius = 3.0;
 }
 
 #ifdef HAVE_ANIMATION
@@ -123,8 +134,8 @@ clearlooks_rc_style_finalize (GObject *object)
 	/* cleanup all the animation stuff */
 	clearlooks_animation_cleanup ();
 	
-	if (G_OBJECT_CLASS (parent_class)->finalize != NULL)
-		G_OBJECT_CLASS (parent_class)->finalize(object);
+	if (G_OBJECT_CLASS (clearlooks_parent_rc_class)->finalize != NULL)
+		G_OBJECT_CLASS (clearlooks_parent_rc_class)->finalize(object);
 }
 #endif
 
@@ -137,7 +148,7 @@ clearlooks_rc_style_class_init (ClearlooksRcStyleClass *klass)
   GObjectClass    *g_object_class = G_OBJECT_CLASS (klass);
 #endif
 
-  parent_class = g_type_class_peek_parent (klass);
+  clearlooks_parent_rc_class = g_type_class_peek_parent (klass);
 
   rc_style_class->parse = clearlooks_rc_style_parse;
   rc_style_class->create_style = clearlooks_rc_style_create_style;
@@ -149,7 +160,7 @@ clearlooks_rc_style_class_init (ClearlooksRcStyleClass *klass)
 }
 
 static guint
-theme_parse_boolean (GtkSettings *settings,
+clearlooks_gtk2_rc_parse_boolean (GtkSettings *settings,
                      GScanner     *scanner,
                      gboolean *retval)
 {
@@ -172,7 +183,7 @@ theme_parse_boolean (GtkSettings *settings,
 }
 
 static guint
-theme_parse_color(GtkSettings  *settings,
+clearlooks_gtk2_rc_parse_color(GtkSettings  *settings,
 		  GScanner     *scanner,
 		  GdkColor     *color)
 {
@@ -189,13 +200,13 @@ theme_parse_color(GtkSettings  *settings,
 }
 
 static guint
-theme_parse_contrast(GtkSettings  *settings,
-		     GScanner     *scanner,
-		     double       *contrast)
+clearlooks_gtk2_rc_parse_double (GtkSettings  *settings,
+                                 GScanner     *scanner,
+                                 gdouble      *val)
 {
   guint token;
 
-  /* Skip 'contrast' */
+  /* Skip 'blah' */
   token = g_scanner_get_next_token(scanner);
 
   token = g_scanner_get_next_token(scanner);
@@ -206,13 +217,13 @@ theme_parse_contrast(GtkSettings  *settings,
   if (token != G_TOKEN_FLOAT)
     return G_TOKEN_FLOAT;
 
-  *contrast = scanner->value.v_float;
+  *val = scanner->value.v_float;
   
   return G_TOKEN_NONE;
 }
 
 static guint
-theme_parse_int (GtkSettings  *settings,
+clearlooks_gtk2_rc_parse_int (GtkSettings  *settings,
 		         GScanner     *scanner,
 		         guint8       *progressbarstyle)
 {
@@ -234,6 +245,64 @@ theme_parse_int (GtkSettings  *settings,
   return G_TOKEN_NONE;
 }
 
+static guint
+clearlooks_gtk2_rc_parse_style (GtkSettings      *settings,
+                                GScanner         *scanner,
+                                ClearlooksStyles *style)
+{
+  guint token;
+
+  g_assert (CL_NUM_STYLES == CL_STYLE_GLOSSY + 1); /* so that people don't forget ;-) */
+
+  /* Skip 'style' */
+  token = g_scanner_get_next_token (scanner);
+
+  token = g_scanner_get_next_token (scanner);
+  if (token != G_TOKEN_EQUAL_SIGN)
+    return G_TOKEN_EQUAL_SIGN;
+
+  token = g_scanner_get_next_token (scanner);
+  
+  switch (token)
+    {
+      case TOKEN_CLASSIC:
+        *style = CL_STYLE_CLASSIC;
+        break;
+      case TOKEN_GLOSSY:
+        *style = CL_STYLE_GLOSSY;
+        break;
+      default:
+        return TOKEN_CLASSIC;
+    }
+
+  return G_TOKEN_NONE;
+}
+
+static guint
+clearlooks_gtk2_rc_parse_dummy (GtkSettings      *settings,
+                                GScanner         *scanner,
+                                gchar            *name)
+{
+  guint token;
+
+  /* Skip option */
+  token = g_scanner_get_next_token (scanner);
+
+  /* print a warning. Isn't there a way to get the string from the scanner? */
+  g_scanner_warn (scanner, "Clearlooks configuration option \"%s\" is not supported and will be ignored.", name);
+
+  /* equal sign */
+  token = g_scanner_get_next_token (scanner);
+  if (token != G_TOKEN_EQUAL_SIGN)
+    return G_TOKEN_EQUAL_SIGN;
+
+  /* eat whatever comes next */
+  token = g_scanner_get_next_token (scanner);
+
+  return G_TOKEN_NONE;
+}
+
+
 static guint
 clearlooks_rc_style_parse (GtkRcStyle *rc_style,
 			   GtkSettings  *settings,
@@ -262,14 +331,12 @@ clearlooks_rc_style_parse (GtkRcStyle *rc_style,
    * same scanner.
    */
 
-  if (!g_scanner_lookup_symbol(scanner, theme_symbols[0].name))
+  if (!g_scanner_lookup_symbol(scanner, clearlooks_gtk2_rc_symbols[0].name))
     {
-      g_scanner_freeze_symbol_table(scanner);
-      for (i = 0; i < G_N_ELEMENTS (theme_symbols); i++)
+      for (i = 0; i < G_N_ELEMENTS (clearlooks_gtk2_rc_symbols); i++)
 	g_scanner_scope_add_symbol(scanner, scope_id,
-				   theme_symbols[i].name,
-				   GINT_TO_POINTER(theme_symbols[i].token));
-      g_scanner_thaw_symbol_table(scanner);
+				   clearlooks_gtk2_rc_symbols[i].name,
+				   GINT_TO_POINTER(clearlooks_gtk2_rc_symbols[i].token));
     }
 
   /* We're ready to go, now parse the top level */
@@ -280,30 +347,44 @@ clearlooks_rc_style_parse (GtkRcStyle *rc_style,
       switch (token)
 	{
 	case TOKEN_SCROLLBARCOLOR:
-	  token = theme_parse_color (settings, scanner, &clearlooks_style->scrollbar_color);
-	  clearlooks_style->has_scrollbar_color = TRUE;
+	  token = clearlooks_gtk2_rc_parse_color (settings, scanner, &clearlooks_style->scrollbar_color);
+	  clearlooks_style->flags |= CL_FLAG_SCROLLBAR_COLOR;
 	  break;
 	case TOKEN_CONTRAST:
-	  token = theme_parse_contrast (settings, scanner, &clearlooks_style->contrast);
-	  break;
-	case TOKEN_SUNKENMENU:
-	  token = theme_parse_int (settings, scanner, &clearlooks_style->sunkenmenubar);
-	  break;
-	case TOKEN_PROGRESSBARSTYLE:
-	  token = theme_parse_int (settings, scanner, &clearlooks_style->progressbarstyle);
+	  token = clearlooks_gtk2_rc_parse_double (settings, scanner, &clearlooks_style->contrast);
+	  clearlooks_style->flags |= CL_FLAG_CONTRAST;
 	  break;
 	case TOKEN_MENUBARSTYLE:
-	  token = theme_parse_int (settings, scanner, &clearlooks_style->menubarstyle);
-	  break;
-	case TOKEN_MENUITEMSTYLE:
-	  token = theme_parse_int (settings, scanner, &clearlooks_style->menuitemstyle);
-	  break;
-	case TOKEN_LISTVIEWITEMSTYLE:
-	  token = theme_parse_int (settings, scanner, &clearlooks_style->listviewitemstyle);
+	  token = clearlooks_gtk2_rc_parse_int (settings, scanner, &clearlooks_style->menubarstyle);
+	  clearlooks_style->flags |= CL_FLAG_MENUBARSTYLE;
 	  break;
 	case TOKEN_ANIMATION:
-	  token = theme_parse_boolean (settings, scanner, &clearlooks_style->animation);
+	  token = clearlooks_gtk2_rc_parse_boolean (settings, scanner, &clearlooks_style->animation);
+	  clearlooks_style->flags |= CL_FLAG_ANIMATION;
 	  break;
+	case TOKEN_STYLE:
+	  token = clearlooks_gtk2_rc_parse_style (settings, scanner, &clearlooks_style->style);
+	  clearlooks_style->flags |= CL_FLAG_STYLE;
+	  break;
+	case TOKEN_RADIUS:
+	  token = clearlooks_gtk2_rc_parse_double (settings, scanner, &clearlooks_style->radius);
+	  clearlooks_style->flags |= CL_FLAG_RADIUS;
+	  break;
+
+	/* stuff to ignore */
+	case TOKEN_SUNKENMENU:
+	  token = clearlooks_gtk2_rc_parse_dummy (settings, scanner, "sunkenmenu");
+	  break;
+	case TOKEN_PROGRESSBARSTYLE:
+	  token = clearlooks_gtk2_rc_parse_dummy (settings, scanner, "progressbarstyle");
+	  break;
+	case TOKEN_MENUITEMSTYLE:
+	  token = clearlooks_gtk2_rc_parse_dummy (settings, scanner, "menuitemstyle");
+	  break;
+	case TOKEN_LISTVIEWITEMSTYLE:
+	  token = clearlooks_gtk2_rc_parse_dummy (settings, scanner, "listviewitemstyle");
+	  break;
+	
 	default:
 	  g_scanner_get_next_token(scanner);
 	  token = G_TOKEN_RIGHT_CURLY;
@@ -328,30 +409,32 @@ clearlooks_rc_style_merge (GtkRcStyle *dest,
 			   GtkRcStyle *src)
 {
 	ClearlooksRcStyle *dest_w, *src_w;
+	ClearlooksRcFlags flags;
 	
-	parent_class->merge (dest, src);
+	clearlooks_parent_rc_class->merge (dest, src);
 	
 	if (!CLEARLOOKS_IS_RC_STYLE (src))
 		return;
 	
 	src_w = CLEARLOOKS_RC_STYLE (src);
 	dest_w = CLEARLOOKS_RC_STYLE (dest);
-	
-	dest_w->contrast          = src_w->contrast;
-	dest_w->sunkenmenubar     = src_w->sunkenmenubar;
-	dest_w->progressbarstyle  = src_w->progressbarstyle;
-	dest_w->menubarstyle      = src_w->menubarstyle;
-	dest_w->menuitemstyle     = src_w->menuitemstyle;
-	dest_w->listviewitemstyle = src_w->listviewitemstyle;
 
-	if (src_w->has_scrollbar_color)
-	{
-		dest_w->has_scrollbar_color = TRUE;
+	flags = (~dest_w->flags) & src_w->flags;
+
+	if (flags & CL_FLAG_STYLE)
+		dest_w->style = src_w->style;
+	if (flags & CL_FLAG_CONTRAST)
+		dest_w->contrast          = src_w->contrast;
+	if (flags & CL_FLAG_MENUBARSTYLE)
+		dest_w->menubarstyle      = src_w->menubarstyle;
+	if (flags & CL_FLAG_SCROLLBAR_COLOR)
 		dest_w->scrollbar_color = src_w->scrollbar_color;
-	}
-	
-	dest_w->animation = src_w->animation;
+	if (flags & CL_FLAG_ANIMATION)
+		dest_w->animation = src_w->animation;
+	if (flags & CL_FLAG_RADIUS)
+		dest_w->radius = src_w->radius;
 
+	dest_w->flags |= src_w->flags;
 }
 
 
